@@ -98,6 +98,7 @@
 ("automatically generated content" "automatisch aangemaakte inhoud")
 ("automatic quotes" "automatische aanhalingstekens")
 ("autosave" "automatisch opslaan")
+("axis" "as")
 ("axiom" "axioma")
 ("b series" "b reeks")
 ("background" "achtergrond")
@@ -142,8 +143,12 @@
 ("both" "beide")
 ("bottom" "bodem")
 ("bottom border" "onderrand")
+("bottom cell border" "onderrand cel")
+("bottom cell padding" "ruimte onderkant cel")
 ("bottom margin" "onderrand")
 ("bottom padding" "ruimte beneden")
+("bottom table border" "onderrand tabel")
+("bottom table padding" "ruimte onderkant tabel")
 ("branch" "tak")
 ("break" "breek af")
 ("breaking" "afbraak")
@@ -631,6 +636,7 @@
 ("insert thick horizontal space" "voeg dikke horizontale spatie in")
 ("insert title" "voeg titel in")
 ("insert value" "voeg waarde in")
+("interactive command" "interaktief kommando")
 ("interfacing" "nieuwe interfaces")
 ("interfacing TeXmacs with other systems" "TeXmacs aan andere systemen koppelen")
 ("interjection" "tussenwerpsel")
@@ -683,6 +689,8 @@
 ("left" "links")
 ("left arrow" "pijl naar links")
 ("left border" "linker rand")
+("left cell border" "linker celrand")
+("left cell padding" "ruimte linkerkant cel")
 ("left margin" "linker rand")
 ("left padding" "ruimte links")
 ("left prime" "linker priem")
@@ -691,6 +699,8 @@
 ("left subscript" "linker index")
 ("left superscript" "linker exponent")
 ("left surrounding" "linker omgeving")
+("left table border" "linker tabelrand")
+("left table padding" "ruimte linkerkant tabel")
 ("legal" "legaal")
 ("lemma" "hulpstelling")
 ("length" "lengte")
@@ -1069,12 +1079,16 @@
 ("right" "rechts")
 ("right arrow" "pijl naar rechts")
 ("right border" "rechter rand")
+("right cell border" "rechter celrand")
+("right cell padding" "ruimte rechterkant cel")
 ("right margin" "rechter rand")
 ("right padding" "ruimte rechts")
 ("right ragged" "rechts")
 ("right subscript" "rechter index")
 ("right superscript" "rechter exponent")
 ("right surrounding" "rechter omgeving")
+("right table border" "rechter tabelrand")
+("right table padding" "ruimte rechterkant tabel")
 ("rigid" "stijf")
 ("rigid horizontal" "stijf horizontaal")
 ("rigid space" "stijve spatie")
@@ -1107,6 +1121,7 @@
 ("save this buffer" "berg deze buffer op")
 ("save verbatim file" "berg verbatim bestand op")
 ("saved" "opgeborgen")
+("scheme command" "scheme kommando")
 ("screen layout" "scherm indeling")
 ("screen margins" "randen op het scherm")
 ("script" "index")
@@ -1244,6 +1259,7 @@
 ("switch to previous" "switch naar vorige")
 ("switching and folding" "switchen en vouwen")
 ("symbol" "symbool")
+("system command" "systeem kommando")
 ("system environment variables" "primitieve omgevingsvariabelen")
 ("tab" "tabulering")
 ("table" "tabel")
@@ -1285,6 +1301,8 @@
 ("this document has been produced using" "dit dokument is tot stand gekomen met behulp van")
 ("this line" "deze regel")
 ("this page" "deze pagina")
+("this page footer" "voet voor deze pagina")
+("this page header" "kop voor deze pagina")
 ("tilda" "slangetje")
 ("tilde" "slangetje")
 ("times" "maal")
@@ -1296,9 +1314,13 @@
 ("tools" "gereedschap")
 ("top" "top")
 ("top border" "bovenrand")
+("top cell border" "bovenrand cel")
+("top cell padding" "ruimte bovenkant cel")
 ("top level" "topniveau")
 ("top margin" "bovenrand")
 ("top padding" "ruimte boven")
+("top table border" "bovenrand tabel")
+("top table padding" "ruimte bovenkant tabel")
 ("translate" "vertaak")
 ("tree" "boom")
 ("tuple" "tupel")
