@@ -98,6 +98,7 @@
 ("automatically generated content" "gnration automatique de contenu")
 ("automatic quotes" "guillemets automatiques")
 ("autosave" "sauvegarde automatique")
+("axis" "axe")
 ("axiom" "axiome")
 ("b series" "srie B")
 ("background" "arrire-plan")
@@ -142,8 +143,12 @@
 ("both" "les deux")
 ("bottom" "en bas")
 ("bottom border" "bordure en bas")
+("bottom cell border" "bordure en bas de la cellule")
+("bottom cell padding" "espacement en bas de la cellule")
 ("bottom margin" "marge en bas")
 ("bottom padding" "espacement en bas")
+("bottom table border" "bordure en bas du tableau")
+("bottom table padding" "espacement en bas du tableau")
 ("branch" "branche")
 ("break" "saut")
 ("breaking" "sauts")
@@ -635,6 +640,7 @@
 ("insert thick horizontal space" "insrer une espace horizontale paisse")
 ("insert title" "insrer titre")
 ("insert value" "insrer valeur")
+("interactive command" "commande interactive")
 ("interfacing" "interfaage")
 ("interfacing TeXmacs with other systems" "interfaage de TeXmacs avec d'autres systmes")
 ("interjection" "interjection")
@@ -687,6 +693,7 @@
 ("left" " gauche")
 ("left arrow" "flche vers la gauche")
 ("left border" "limite  gauche")
+("left cell border" "bordure de la cellule  gauche")
 ("left margin" "marge  gauche")
 ("left padding" "espacement  gauche")
 ("left prime" "prime  gauche")
@@ -695,6 +702,7 @@
 ("left subscript" "indice  gauche")
 ("left superscript" "exposant  gauche")
 ("left surrounding" "entourage  gauche")
+("left table border" "bordure du tableau  gauche")
 ("legal" "lgal")
 ("lemma" "lemme")
 ("length" "longueur")
@@ -1078,6 +1086,7 @@ styles standards de TeXmacs")
 ("right" " droite")
 ("right arrow" "flche vers la droite")
 ("right border" "limite  droite")
+("right cell border" "bordure de la cellule  droite")
 ("right margin" "marge  droite")
 ("right padding" "espacement  droite")
 ("right page margin" "marge de page  droite")
@@ -1085,6 +1094,7 @@ styles standards de TeXmacs")
 ("right subscript" "indice  droite")
 ("right superscript" "exposant  droite")
 ("right surrounding" "entourage  droite")
+("right table border" "bordure du tableau  droite")
 ("rigid" "fixe")
 ("rigid horizontal" "horizontal fixe")
 ("rigid space" "espace fixe")
@@ -1118,6 +1128,7 @@ styles standards de TeXmacs")
 ("save this buffer" "sauvegarder ce tampon")
 ("save verbatim file" "sauvegarder fichier verbatim")
 ("saved" "sauvegard")
+("scheme command" "commande Scheme")
 ("screen layout" "apparence sur cran")
 ("screen margins" "marges sur l'cran")
 ("script" "indice")
@@ -1255,6 +1266,7 @@ styles standards de TeXmacs")
 ("switch to previous" "passer au prcdant")
 ("switching and folding" "commuter et plier")
 ("symbol" "symbole")
+("system command" "commande systme")
 ("system environment variables" "variables d'environnement systmes")
 ("tab" "tabulation")
 ("table" "tableau")
@@ -1296,6 +1308,8 @@ styles standards de TeXmacs")
 ("this document has been produced using" "ce document a t conu avec")
 ("this line" "cette ligne")
 ("this page" "cette page")
+("this page footer" "en bas de cette page")
+("this page header" "en-tte pour cette page")
 ("tilda" "tilde")
 ("tilde" "tilde")
 ("times" "fois")
@@ -1307,9 +1321,13 @@ styles standards de TeXmacs")
 ("tools" "outils")
 ("top" "en haut")
 ("top border" "bordure en haut")
+("top cell border" "bordure en haut de la cellule")
+("top cell padding" "espacement en haut de la cellule")
 ("top level" "niveau suprieur")
 ("top margin" "marge en haut")
 ("top padding" "espacement en haut")
+("top table border" "bordure en haut du tableau")
+("top table padding" "espacement en haut du tableau")
 ("translate" "traduire")
 ("tree" "arbre")
 ("tuple" "tuplet")
