@@ -15,15 +15,10 @@
 (texmacs-module (convert html htmltm)
   (:use
     (convert tools tmlength) (convert tools tmcolor)
-    (convert tools tmtable) (convert tools stm)
+    (convert tools old-tmtable) (convert tools stm)
     (convert tools sxml)  (convert tools sxhtml)
     (convert tools environment)
-    (convert tools xmltm) (convert mathml mathtm))
-  (:export 
-    parse-html-snippet parse-html-document
-    html->texmacs
-    htmltm-as-serial ;; for htmltm-test
-    ))
+    (convert tools xmltm) (convert mathml mathtm)))
 
 (define (assoc-string-ci key alist)
   (list-find alist (lambda (pair) (string-ci=? key (car pair)))))
@@ -99,7 +94,7 @@
 
 (define (table-formats env a c)
   ;; As a convention, global properties are placed at the end of the list.
-  ;; Remember that tmtable->tm reverses the list of table formats.
+  ;; Remember that tmtable->stm reverses the list of table formats.
   (append (table-content-formats env c)
 	  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	  ;; TODO: table cellspacing and cellpadding
@@ -389,7 +384,7 @@
 (define (cleanup-root env root)
   (sxml-set-content root (htmltm-space-mixed env (sxml-content root))))
 
-(define (htmltm-as-serial root)
+(tm-define (htmltm-as-serial root)
   ;; As htmltm, but returns a serial node.
   ;; Actually also initializes the dynamic enviroment.
   ;; FIXME: move the htmlinitialization elsewhere for symmetry with htmltm.
@@ -531,10 +526,10 @@
 ;; Interface
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (parse-html-snippet s)
+(tm-define (parse-html-snippet s)
   (htmltm-parse s))
 
-(define (parse-html-document s)
+(tm-define (parse-html-document s)
   `(!file ,(htmltm-parse s)))
 
 (tm-define (html->texmacs html)
@@ -544,7 +539,8 @@
 	 (body (if snippet? html (cadr html)))
 	 (tm (htmltm-as-serial (sxhtml-correct-table body))))
     (if snippet? tm
-	(let* ((doc (tree-simplify (stree->tree (stm-unary-document tm))))
-	       (body (tree1 'body doc))
-	       (style (tree1 'style (string->tree "browser"))))
-	  (tree2 'document body style)))))
+	(let* ((aux (stm-unary-document tm))
+	       (doc (tree->stree (tree-simplify (stree->tree aux))))
+	       (body `(body ,doc))
+	       (style `(style "browser")))
+	  `(document ,body ,style)))))
