@@ -161,7 +161,7 @@ edit_dynamic_rep::go_to_argument (path p, bool start_flag) {
     if ((!drd->is_accessible_child (t, i)) &&
 	(!inactive) && (!in_source ()))
       {
-	ins_unary (path_up (p), INACTIVE);
+	insert_node (path_up (p) * 0, INACTIVE);
 	p= path_up (p) * path (0, i);
       }
     if (start_flag) go_to_start (p);
@@ -214,7 +214,7 @@ edit_dynamic_rep::remove_argument (path p, bool forward) {
 	  bool old_locked= env_locked; env_locked= true;
 	  remove (p, d);
 	  if ((d == n) && is_mod_active_once (subtree (et, path_up (p, 2)))) {
-	    rem_unary (path_up (p, 2));
+	    remove_node (path_up (p, 2) * 0);
 	    go_to_border (path_up (p, 2), forward);
 	  }
 	  else if (forward) go_to_argument (path_up (p) * i, true);
@@ -258,6 +258,31 @@ edit_dynamic_rep::remove_argument (path p, bool forward) {
   else go_to_argument (path_up (p) * (i-1), false);
 }
 
+void
+edit_dynamic_rep::remove_argument (bool forward) {
+  path p= find_dynamic (tp);
+  if (nil (p)) return;
+  if (p == tp) p= find_dynamic (path_up (tp));
+  if (nil (p)) return;
+
+  tree t= subtree (et, path_up (p));
+  int i= last_item (p), n= N(t), d= 1;
+  if ((!in_source ()) || drd->contains (as_string (L(t)))) {
+    if (forward) do i++; while (i<=n && !drd->insert_point (L(t), i, n));
+    else while (i>=0 && !drd->insert_point (L(t), i, n)) i--;
+    if ((i<0) || (i>n)) return;
+    while (i>=d && !drd->correct_arity (L(t), n-d)) d++;
+    if (i<d || n<=d || !drd->insert_point (L(t), i-d, n-d)) return;
+  }
+  else {
+    if (forward) i++;
+    if (i<d || n<=d) return;
+  }
+  path q= path_up (p) * (i-d);
+  remove (q, d);
+  go_to_argument (q, forward);
+}
+
 /******************************************************************************
 * Backspace and delete
 ******************************************************************************/
@@ -357,10 +382,7 @@ edit_dynamic_rep::insert_with (path p, string var, tree val) {
   }
   else if ((rp < p) && is_func (subtree (et, path_up (p)), WITH))
     insert_with (path_up (p), var, val);
-  else {
-    ins_unary (p, WITH);
-    insert (p * 0, copy (tree (WITH, var, val)));
-  }
+  else insert_node (p * 2, copy (tree (WITH, var, val)));
 }
 
 void
@@ -371,7 +393,7 @@ edit_dynamic_rep::remove_with (path p, string var) {
     for (i=0; i<n; i+=2)
       if (st[i] == var) {
 	remove (p * i, 2);
-	if (n == 2) rem_unary (p);
+	if (n == 2) remove_node (p * 0);
 	return;
       }
   }
@@ -407,7 +429,7 @@ edit_dynamic_rep::make_mod_active (tree_label l) {
   else {
     path p= path_up (tp);
     if (is_atomic (subtree (et, p))) p= path_up (p);
-    if (rp < p) ins_unary (p, l);
+    if (rp < p) insert_node (p * 0, l);
   }
 }
 
@@ -424,10 +446,7 @@ edit_dynamic_rep::insert_style_with (path p, string var, string val) {
       }
     insert (path_up (p) * (n-1), tree (STYLE_WITH, copy (var), copy (val)));
   }
-  else {
-    ins_unary (p, STYLE_WITH);
-    insert (p * 0, tree (STYLE_WITH, copy (var), copy (val)));
-  }
+  else insert_node (p * 2, copy (tree (STYLE_WITH, var, val)));
 }
 
 void
