@@ -19,33 +19,94 @@
   "initialize_glue_basic"
 
   (texmacs-version-release texmacs_version (string string))
+  (version-before? version_inf (bool string string))
   (os-win32? os_win32 (bool))
   (win32-display win32_display (void string))
+  (scheme-dialect scheme_dialect (string))
+  (get-texmacs-path get_texmacs_path (string))
+  (plugin-list plugin_list (scheme_tree))
+  (get-font-type get_font_type (int))
+  (set-font-type set_font_type (void int))
+  (font-exists-in-tt? tt_font_exists (bool string))
+  (eval-system eval_system (string string))
+  (var-eval-system var_eval_system (string string))
+  (get-locale-language get_locale_language (string))
+  (texmacs-time texmacs_time (int))
+  (bench-print bench_print (void string))
+  (bench-print-all bench_print (void))
+  (system-wait system_wait (void string string))
+  (set-bibtex-command set_bibtex_command (void string))
+  (math-symbol-group math_symbol_group (string string))
+  (math-group-members math_group_members (array_string string))
+  (math-symbol-type math_symbol_type (string string))
+  (object->command as_command (command object))
 
+  ;; routines for trees
   (tree->stree tree_to_scheme_tree (scheme_tree tree))
   (stree->tree scheme_tree_to_tree (tree scheme_tree))
   (tree->string coerce_tree_string (string tree))
-  (tree-get-label L (tree_label tree))
-  (tree-get-children A (array_tree tree))
   (string->tree coerce_string_tree (tree string))
-  (tree0 tree (tree tree_label))
-  (tree1 tree (tree tree_label tree))
-  (tree2 tree (tree tree_label tree tree))
-  (tree3 tree (tree tree_label tree tree tree))
-  (tree-ip obtain_ip (path tree))
+  (tm->tree tree (tree content))
   (tree-atomic? is_atomic (bool tree))
   (tree-compound? is_compound (bool tree))
+  (tree-label L (tree_label tree))
+  (tree-children A (array_tree tree))
   (tree-arity N (int tree))
-  (tree-ref tree_ref (tree tree int))
-  (tree-set! tree_set (void tree int tree))
-  (tree-range tree_range (tree tree int int))
+  (tree-child tree_ref (tree tree int))
+  (tree-set-child! tree_set (void tree int tree))
+  (tree-ip obtain_ip (path tree))
   (subtree subtree (tree tree path))
+  (tree-range tree_range (tree tree int int))
   (tree-copy copy (tree tree))
   (tree-append tree_append (tree tree tree))
   (tree-label-extension? is_extension (bool tree_label))
   (tree-multi-paragraph? is_multi_paragraph (bool tree))
   (tree-simplify simplify_correct (tree tree))
+  (tree-accessible-child? is_accessible_child (bool tree int))
+  (tree-accessible-children accessible_children (array_tree tree))
+  (tree-load-inclusion load_inclusion (tree url))
 
+  ;; paths
+  (path-inf? path_inf (bool path path))
+  (path-inf-eq? path_inf_eq (bool path path))
+  (path-less? path_less (bool path path))
+  (path-less-eq? path_less_eq (bool path path))
+  (path-start start (path content path))
+  (path-end end (path content path))
+  (path-next next_valid (path content path))
+  (path-previous previous_valid (path content path))
+  (path-next-word next_word (path content path))
+  (path-previous-word previous_word (path content path))
+  (path-next-tag next_tag (path content path scheme_tree))
+  (path-previous-tag previous_tag (path content path scheme_tree))
+  (path-next-argument next_argument (path content path))
+  (path-previous-argument previous_argument (path content path))
+
+  ;; routines for strings
+  (string-number? is_double (bool string))
+  (string-search-forwards search_forwards (int string int string))
+  (string-search-backwards search_backwards (int string int string))
+  (string-replace replace (string string string string))
+  (string-locase-alpha? is_locase_alpha (bool string))
+  (upcase-first upcase_first (string string))
+  (locase-first locase_first (string string))
+  (upcase-all upcase_all (string string))
+  (locase-all locase_all (string string))
+  (string-union string_union (string string string))
+  (string-minus string_minus (string string string))
+  (escape-generic escape_generic (string string))
+  (escape-verbatim escape_verbatim (string string))
+  (utf8->cork utf8_to_cork (string string))
+  (cork->utf8 cork_to_utf8 (string string))
+  (utf8->html utf8_to_html (string string))
+  (tm->xml-name tm_to_xml_name (string string))
+  (old-tm->xml-cdata old_tm_to_xml_cdata (string string))
+  (tm->xml-cdata tm_to_xml_cdata (object string))
+  (xml-name->tm xml_name_to_tm (string string))
+  (old-xml-cdata->tm old_xml_cdata_to_tm (string string))
+  (xml-unspace xml_unspace (string string bool bool))
+
+  ;; further conversion routines for trees and strings
   (parse-texmacs texmacs_document_to_tree (tree string))
   (serialize-texmacs tree_to_texmacs (string tree))
   (parse-texmacs-snippet texmacs_to_tree (tree string))
@@ -62,33 +123,8 @@
   (parse-xml parse_xml (scheme_tree string))
   (parse-html parse_html (scheme_tree string))
   (upgrade-tmml tmml_upgrade (tree scheme_tree))
-  (get-texmacs-path get_texmacs_path (string))
-  (object->command as_command (command object))
-
-  (scheme-dialect scheme_dialect (string))
-  (cursor-start start (path content path))
-  (cursor-end end (path content path))
-  (support-ec-fonts? support_ec_fonts (bool))
-  (support-tt-fonts? ft_present (bool))
-  (use-ec-fonts? use_ec_fonts (bool))
-  (use-tt-fonts? use_tt_fonts (bool))
-  (set-font-type set_font_type (void int))
-  (font-exists-in-tt? tt_font_exists (bool string))
-  (texmacs-time texmacs_time (int))
-  (bench-print bench_print (void string))
-  (bench-print-all bench_print (void))
-  (plugin-list plugin_list (scheme_tree))
-  (eval-system eval_system (string string))
-  (var-eval-system var_eval_system (string string))
-  (get-locale-language get_locale_language (string))
-  (escape-quotes escape_quotes (string string))
-  (escape-generic escape_generic (string string))
-  (escape-verbatim escape_verbatim (string string))
-  (system-wait system_wait (void string string))
-  (math-symbol-type math_symbol_type (string string))
-  (string-number? is_double (bool string))
-  (set-bibtex-command set_bibtex_command (void string))
 
+  ;; routines for urls
   (string->url url (url string))
   (url url (url string string))
   (url-system url_system (url string))
@@ -104,7 +140,6 @@
   (url-concat? is_concat (bool url))
   (url-or? is_or (bool url))
   (url-ref url_ref (url url int))
-
   (url-tail tail (url url))
   (url-suffix suffix (string url))
   (url-glue glue (url url string))
@@ -122,13 +157,11 @@
   (url-exists-in-tex? exists_in_tex (bool url))
   (url-concretize concretize (string url))
   (url-materialize materialize (string url string))
-
   (url-test? is_of_type (bool url string))
   (url-regular? is_regular (bool url))
   (url-directory? is_directory (bool url))
   (url-link? is_symbolic_link (bool url))
   (url-newer? is_newer (bool url url))
-
   (url-temp url_temp (url))
   (string-save string_save (void string url))
   (string-load string_load (string url))
@@ -137,26 +170,11 @@
   (system-append append (void url url))
   (system-remove remove (void url))
   (system-mkdir mkdir (void url))
+  (system-search-score search_score (int url array_string))
+  (system-1 system (void string url))
+  (system-2 system (void string url url))
 
-  (string-search-forwards search_forwards (int string int string))
-  (string-search-backwards search_backwards (int string int string))
-  (string-replace replace (string string string string))
-  (string-slash slash (string string))
-  (string-unslash unslash (string string))
-  (string-locase-alpha? is_locase_alpha (bool string))
-  (upcase-first upcase_first (string string))
-  (locase-first locase_first (string string))
-  (upcase-all upcase_all (string string))
-  (locase-all locase_all (string string))
-  (utf8->cork utf8_to_cork (string string))
-  (cork->utf8 cork_to_utf8 (string string))
-  (utf8->html utf8_to_html (string string))
-  (tm->xml-name tm_to_xml_name (string string))
-  (tm->xml-cdata tm_to_xml_cdata (string string))
-  (xml-name->tm xml_name_to_tm (string string))
-  (xml-cdata->tm xml_cdata_to_tm (string string))
-  (xml-unspace xml_unspace (string string bool bool))
-
+  ;; connections to extern systems
   (connection-declared? connection_declared (bool string))
   (connection-status connection_status (int string string))
   (connection-start connection_start (string string string bool))
@@ -165,11 +183,7 @@
   (connection-write connection_write (void string string content))
   (connection-read connection_read (tree string string string))
 
-  (path-inf? path_inf (bool path path))
-  (path-inf-eq? path_inf_eq (bool path path))
-  (path-less? path_less (bool path path))
-  (path-less-eq? path_less_eq (bool path path))
-
+  ;; widgets
   (widget-hlist horizontal_list (widget array_widget))
   (widget-named-hlist horizontal_list (widget array_widget array_string))
   (widget-vlist vertical_list (widget array_widget))
