@@ -33,6 +33,7 @@ struct object {
   object ();
   object (bool b);
   object (int i);
+  object (char* s);
   object (string s);
   object (tree t);
   object (path p);
@@ -45,6 +46,7 @@ bool operator == (object obj1, object obj2);
 bool operator != (object obj1, object obj2);
 
 object null_object ();
+object symbol_object (string s);
 object cons (object obj1, object obj2);
 object car (object obj);
 object cdr (object obj);
@@ -60,6 +62,7 @@ bool is_list (object obj);
 bool is_bool (object obj);
 bool is_int (object obj);
 bool is_string (object obj);
+bool is_symbol (object obj);
 bool is_tree (object obj);
 bool is_path (object obj);
 bool is_url (object obj);
@@ -67,12 +70,12 @@ bool is_url (object obj);
 bool as_bool (object obj);
 int as_int (object obj);
 string as_string (object obj);
+string as_symbol (object obj);
 tree as_tree (object obj);
 scheme_tree as_scheme_tree (object obj);
 path as_path (object obj);
 url as_url (object obj);
 command as_command (object obj);
-command as_command (string s);
 #ifdef WIDGET_H // FIXME: dirty hack
 #include "Widget/make_widget.hpp"
 widget as_widget (object obj);
@@ -84,6 +87,9 @@ tree   stree_to_tree (object obj);
 tree   content_to_tree (object obj);
 object string_to_object (string s);
 string object_to_string (object obj);
+object scheme_cmd (char* s);
+object scheme_cmd (string s);
+object scheme_cmd (object cmd);
 
 object eval (char* expr);
 object eval (string expr);
