@@ -16,6 +16,7 @@
 #include "hashmap.hpp"
 #include "timer.hpp"
 #include "merge_sort.hpp"
+#include "data_cache.hpp"
 
 #include <stddef.h>
 #include <stdio.h>
@@ -41,8 +42,20 @@ load_string (url u, string& s, bool fatal) {
   if (!is_rooted_name (r)) r= resolve (r);
   bool err= !is_rooted_name (r);
   if (!err) {
-    bench_start ("load file");
     string name= concretize (r);
+
+    // File contents in cache?
+    bool file_flag= do_cache_file (name);
+    bool doc_flag= do_cache_doc (name);
+    string cache_type= doc_flag? string ("doc_cache"): string ("file_cache");
+    if (doc_flag) cache_load ("doc_cache");
+    if (is_cached (cache_type, name)) {
+      s= cache_get (cache_type, name) -> label;
+      return false;
+    }
+    // End caching
+
+    bench_start ("load file");
     char* _name= as_charp (name);
 #ifdef OS_WIN32
     FILE* fin= _fopen (_name, "rb");
@@ -67,37 +80,17 @@ load_string (url u, string& s, bool fatal) {
     }
     delete[] _name;
     bench_cumul ("load file");
-  }
-  if (err && fatal)
-    fatal_error (as_string (u) * " not readable", "load_string");
-  return err;
-}
 
-/*
-#include <fstream>
-bool
-load_string (url u, string& s, bool fatal) {
-  string name= concretize (u);
-  char* _name= as_charp (name);
-  bool err= !is_rooted_name (u);
-  if (!err) {
-    cout << "opening " << name << "\n";
-    ifstream fin (_name);
-    cout << "opened\n";
-    if (!fin) err= true;
-    if (!err) {
-      char c;
-      while (fin.get (c)) s << c;
-      if (!fin.eof ()) err= true;
-    }
+    // Cache file contents
+    if (!err && N(s) <= 10000)
+      if (file_flag || doc_flag)
+	cache_set (cache_type, name, s);
+    // End caching
   }
-  cout << "read " << s << "\n";
   if (err && fatal)
     fatal_error (as_string (u) * " not readable", "load_string");
-  delete[] _name;
   return err;
 }
-*/
 
 bool
 save_string (url u, string s, bool fatal) {
@@ -121,6 +114,15 @@ save_string (url u, string s, bool fatal) {
       fclose (fout);
     }
     delete[] _name;
+
+    // Cache file contents
+    bool file_flag= do_cache_file (name);
+    bool doc_flag= do_cache_doc (name);
+    string cache_type= doc_flag? string ("doc_cache"): string ("file_cache");
+    if (!err && N(s) <= 10000)
+      if (file_flag || doc_flag)
+	cache_set (cache_type, name, s);
+    // End caching
   }
   if (err && fatal)
     fatal_error (as_string (u) * " not writeable", "save_string");
@@ -134,9 +136,20 @@ save_string (url u, string s, bool fatal) {
 static bool
 get_attributes (url name, struct stat* buf, bool link_flag=false) {
   // cout << "Stat " << name << LF;
+  string name_s= concretize (name);
+
+  // Stat result in cache?
+  if (is_cached ("stat_cache.scm", name_s)) {
+    string r= cache_get ("stat_cache.scm", name_s) -> label;
+    if (r == "#f") return true;
+    buf->st_mode= ((unsigned int) as_int (r));
+    return false;
+  }
+  // End caching
+
   bench_start ("stat");
   bool flag;
-  char* temp= as_charp (concretize (name));
+  char* temp= as_charp (name_s);
 #ifdef OS_WIN32
   flag= _stat (temp, buf);
 #else
@@ -147,6 +160,18 @@ get_attributes (url name, struct stat* buf, bool link_flag=false) {
   // flag= (link_flag? lstat (temp, buf): stat (temp, buf));
   delete[] temp;
   bench_cumul ("stat");
+
+  // Cache stat results
+  if (flag) {
+    if (do_cache_stat_fail (name_s))
+      cache_set ("stat_cache.scm", name_s, "#f");
+  }
+  else {
+    if (do_cache_stat (name_s))
+      cache_set ("stat_cache.scm", name_s, as_string ((int) buf->st_mode));
+  }
+  // End caching
+
   return flag;
 }
 
@@ -199,6 +224,8 @@ bool
 is_newer (url which, url than) {
   struct stat which_stat;
   struct stat than_stat;
+  if (is_cached ("stat_cache.scm", concretize (which))) return false;
+  if (is_cached ("stat_cache.scm", concretize (than))) return false;
   if (get_attributes (which, &which_stat, true)) return false;
   if (get_attributes (than , &than_stat , true)) return false;
   return which_stat.st_mtime > than_stat.st_mtime;
@@ -223,13 +250,33 @@ url_temp (string suffix) {
 * Reading directories
 ******************************************************************************/
 
+static array<string>
+cache_dir_get (string dir) {
+  tree t= cache_get ("dir_cache.scm", dir);
+  array<string> a (N(t));
+  for (int i=0; i<N(t); i++) a[i]= t[i]->label;
+  return a;
+}
+
+static void
+cache_dir_set (string dir, array<string> a) {
+  tree t (TUPLE, N(a));
+  for (int i=0; i<N(a); i++) t[i]= a[i];
+  cache_set ("dir_cache.scm", dir, t);
+}
+
 array<string>
 read_directory (url u, bool& error_flag) {
   // cout << "Directory " << u << LF;
   u= resolve (u, "dr");
   if (is_none (u)) return array<string> ();
   string name= concretize (u);
+
+  // Directory contents in cache?
+  if (is_cached ("dir_cache.scm", name))
+    return cache_dir_get (name);
   bench_start ("read directory");
+  // End caching
 
   DIR* dp;
   char* temp= as_charp (name);
@@ -248,10 +295,92 @@ read_directory (url u, bool& error_flag) {
   (void) closedir (dp);
   merge_sort (dir);
 
+  // Caching of directory contents
   bench_cumul ("read directory");
+  if (do_cache_dir (name))
+    cache_dir_set (name, dir);
+  // End caching
+
   return dir;
 }
 
+/******************************************************************************
+* Searching text in the documentation
+******************************************************************************/
+
+static array<int>
+search (string what, string in) {
+  int i= 0, n= N(what);
+  array<int> matches;
+  if (n == 0) return matches;
+  while (true) {
+    int pos= search_forwards (what, i, in);
+    if (pos == -1) return matches;
+    matches << pos;
+    i= pos+1;
+  }
+}
+
+static bool
+precedes (string in, int pos, string what) {
+  return pos >= N(what) && in (pos-N(what), pos) == what;
+}
+
+static int
+compute_score (string what, string in, int pos, string suf) {
+  int score= 1;
+  if (pos > 0 && !is_iso_alpha (in [pos-1]))
+    if (pos + N(what) + 1 < N(in) && !is_iso_alpha (in [pos+N(what)]))
+      score *= 10;
+  if (suf == "tm") {
+    if (precedes (in, pos, "<")) score= 0;
+    else if (precedes (in, pos, "<\\")) score= 0;
+    else if (precedes (in, pos, "<|")) score= 0;
+    else if (precedes (in, pos, "</")) score= 0;
+    else if (precedes (in, pos, "compound|")) score= 0;
+    else if (precedes (in, pos, "<name|")) score *= 10;
+    else if (precedes (in, pos, "<tmstyle|")) score *= 10;
+    else if (precedes (in, pos, "<tmdtd|")) score *= 10;
+    else if (precedes (in, pos, "<explain-macro|")) score *= 10;
+    else if (precedes (in, pos, "<var-val|")) score *= 10;
+  }
+  else if (suf == "scm") {
+    if (precedes (in, pos, "define ")) score *= 10;
+    else if (precedes (in, pos, "define-public ")) score *= 10;
+    else if (precedes (in, pos, "define (")) score *= 10;
+    else if (precedes (in, pos, "define-public (")) score *= 10;
+    else if (precedes (in, pos, "define-macro ")) score *= 10;
+    else if (precedes (in, pos, "define-public-macro ")) score *= 10;
+    else if (precedes (in, pos, "define-macro (")) score *= 10;
+    else if (precedes (in, pos, "define-public-macro (")) score *= 10;
+  }
+  return score;
+}
+
+static int
+compute_score (string what, string in, array<int> pos, string suf) {
+  int score= 0, i= 0, n= N(pos);
+  for (i=0; i<n; i++)
+    score += compute_score (what, in, pos[i], suf);
+  return score;
+}
+
+int
+search_score (url u, array<string> a) {
+  string in, suf= suffix (u);
+  if (load_string (u, in, false)) return 0;
+  in= locase_all (in);
+  int i, score= 1, n= N(a);
+  for (i=0; i<n; i++) {
+    string what= locase_all (a[i]);
+    array<int> pos= search (what, in);
+    score *= compute_score (what, in, pos, suf);
+    if (score == 0) return 0;
+    if (score > 1000000) score= 1000000;
+  }
+  return score;
+}
+
 /******************************************************************************
 * Miscellaneous
 ******************************************************************************/
