@@ -12,6 +12,7 @@
 
 #include "env.hpp"
 #include "PsDevice/page_type.hpp"
+#include "typesetter.hpp"
 
 /******************************************************************************
 * Retrieving the page size
@@ -72,8 +73,8 @@ initialize_default_var_type () {
   var_type (PAGE_TYPE)         = Env_Fixed;
   var_type (PAGE_BREAKING)     = Env_Fixed;
   var_type (PAGE_FLEXIBILITY)  = Env_Fixed;
-  var_type (PAGE_WIDTH)        = Env_Fixed;
-  var_type (PAGE_HEIGHT)       = Env_Fixed;
+  var_type (PAGE_WIDTH)        = Env_Page_Extents;
+  var_type (PAGE_HEIGHT)       = Env_Page_Extents;
   var_type (PAGE_WIDTH_MARGIN) = Env_Page;
   var_type (PAGE_SCREEN_MARGIN)= Env_Page;
   var_type (PAGE_NR)           = Env_Page;
@@ -97,15 +98,21 @@ initialize_default_var_type () {
   var_type (PAGE_THIS_HEADER)  = Env_Page;
   var_type (PAGE_THIS_FOOTER)  = Env_Page;
   var_type (PAGE_FNOTE_SEP)    = Env_Page;
-  var_type (POINT_STYLE)       = Env_Point_Style;
   var_type (PAGE_FNOTE_BARLEN) = Env_Page;
   var_type (PAGE_FLOAT_SEP)    = Env_Page;
   var_type (PAGE_MNOTE_SEP)    = Env_Page;
   var_type (PAGE_MNOTE_WIDTH)  = Env_Page;
 
+  var_type (POINT_STYLE)       = Env_Point_Style;
   var_type (LINE_WIDTH)        = Env_Line_Width;
+  var_type (DASH_STYLE)        = Env_Dash_Style;
+  var_type (DASH_STYLE_UNIT)   = Env_Dash_Style_Unit;
+  var_type (FILL_COLOR)        = Env_Fill_Color;
+  var_type (LINE_ARROWS)       = Env_Line_Arrows;
+  var_type (TEXTAT_HALIGN)     = Env_Textat_Halign;
+  var_type (TEXTAT_VALIGN)     = Env_Textat_Valign;
   var_type (GR_FRAME)          = Env_Frame;
-  var_type (GR_CLIP)           = Env_Clipping;
+  var_type (GR_GEOMETRY)       = Env_Geometry;
   var_type (GR_GRID)           = Env_Grid;
   var_type (GR_GRID_ASPECT)    = Env_Grid_Aspect;
   var_type (GR_EDIT_GRID)        = Env_Grid;
@@ -264,8 +271,18 @@ edit_env_rep::update_font () {
 
 void
 edit_env_rep::update_color () {
-  string s= get_string (COLOR);
-  col= dis->get_color (s);
+  string c= get_string (COLOR);
+  string fc= get_string (FILL_COLOR);
+  if (c == "none") {
+    if (fc == "none") fill_mode= FILL_MODE_NOTHING;
+    else fill_mode= FILL_MODE_INSIDE;
+  }
+  else {
+    if (fc == "none") fill_mode= FILL_MODE_NONE;
+    else fill_mode= FILL_MODE_BOTH;
+  }
+  col= dis->get_color (c);
+  fill_color= dis->get_color (fc);
 }
 
 void
@@ -293,35 +310,53 @@ edit_env_rep::update_language () {
   }
 }
 
+void
+edit_env_rep::update_geometry () {
+  tree t= env [GR_GEOMETRY];
+  gw= as_length ("1par");
+  gh= as_length ("0.6par");
+  gvalign= as_string ("center");
+  if (is_tuple (t, "geometry", 2) || is_tuple (t, "geometry", 3)) {
+    if (is_length (as_string (t[1]))) gw= as_length (t[1]);
+    if (is_length (as_string (t[2]))) gh= as_length (t[2]);
+    if (is_tuple (t, "geometry", 3))
+      gvalign= as_string (t[3]);
+  }
+  update_frame ();
+}
+
 void
 edit_env_rep::update_frame () {
   tree t= env [GR_FRAME];
+  SI yinc= gvalign == "top"    ? - gh
+	 : gvalign == "bottom" ? 0
+	 : - gh / 2;
   if (is_tuple (t, "scale", 2) && is_func (t[2], TUPLE, 2)) {
     SI magn= as_length (t[1]);
     SI x   = as_length (t[2][0]);
     SI y   = as_length (t[2][1]);
-    fr= scaling (magn, point (x, y));
+    if (gvalign == "top") yinc += as_length ("1ex");
+    fr= scaling (magn, point (x, y + yinc));
   }
   else {
     SI cm   = as_length (string ("1cm"));
     SI par  = as_length (string ("1par"));
     SI yfrac= as_length (string ("1yfrac"));
-    fr= scaling (cm, point (par >> 1, yfrac));
+    fr= scaling (cm, point (par >> 1, yfrac + yinc));
   }
-}
-
-void
-edit_env_rep::update_clipping () {
-  tree t= env [GR_CLIP];
-  if (is_tuple (t, "clip", 2) &&
-      is_func (t[1], TUPLE, 2) &&
-      is_func (t[2], TUPLE, 2)) {
-    clip_lim1= as_point (t[1]);
-    clip_lim2= as_point (t[2]);
+  point p0= fr (as_point (tuple ("0par", "0par")));
+  if (gvalign == "top") {
+    clip_lim1= fr [point (p0[0], p0[1] - gh + as_length ("1ex"))];
+    clip_lim2= fr [point (p0[0] + gw, p0[1] + as_length ("1ex"))];
+  }
+  else
+  if (gvalign == "bottom") {
+    clip_lim1= fr [point (p0[0], p0[1])];
+    clip_lim2= fr [point (p0[0] + gw, p0[1] + gh)];
   }
   else {
-    clip_lim1= as_point (tuple ("0par", "-0.3par"));
-    clip_lim2= as_point (tuple ("1par", "0.3par"));
+    clip_lim1= fr [point (p0[0], p0[1] - gh/2)];
+    clip_lim2= fr [point (p0[0] + gw, p0[1] + gh/2)];
   }
 }
 
@@ -362,6 +397,55 @@ edit_env_rep::update_src_close () {
   else if (s == "repeat") src_close= CLOSE_REPEAT;
 }
 
+void
+edit_env_rep::update_dash_style () {
+  tree t= env [DASH_STYLE];
+  dash_style= array<bool>(0);
+  if (is_string (t)) {
+    string s= as_string (t);
+    if (s == "none") ;
+  }
+  else
+  if (is_tuple (t)) {
+    int i, n= N(t);
+    dash_style= array<bool> (n);
+    for (i=0; i<n; i++) {
+      dash_style[i]= true;
+      if (t[i] == "0") dash_style[i]= false;
+    }
+  }
+}
+
+/*FIXME: Currently, the line-arrows property is evaluated
+  only in the context of the variables which appear before
+  it in the <with>. For example :
+
+  - <with|color|blue|<with|color|green|line-arrows|<line|...>|...>
+    draws green objects with green line arrows;
+
+  - while <with|color|blue|<with|line-arrows|<line|...>|color|green|...>
+    draws green objects with blue line arrows.
+ */
+void
+edit_env_rep::update_line_arrows () {
+  tree t= env [LINE_ARROWS];
+  line_arrows= array<box>(2);
+  if (is_string (t)) {
+    string s= as_string (t);
+    if (s == "none") ;
+  }
+  else
+  if (is_tuple (t) && N(t)<=2 && N(t)>0) {
+    array<box> b (2);
+    b[1]= t[0]=="" ? box () : typeset_as_box (this, t[0], path(0));
+    if (N(t)>=2) {
+      b[0]= b[1];
+      b[1]= t[1]=="" ? box () : typeset_as_box (this, t[1], path (0));
+    }
+    line_arrows= b;
+  }
+}
+
 void
 edit_env_rep::update () {
   magn           = get_double (MAGNIFICATION);
@@ -376,10 +460,15 @@ edit_env_rep::update () {
   update_language ();
   update_font ();
 
+  update_geometry ();
   update_frame ();
-  update_clipping ();
   point_style= get_string (POINT_STYLE);
   lw= get_length (LINE_WIDTH);
+  update_dash_style ();
+  dash_style_unit= get_length (DASH_STYLE_UNIT);
+  update_line_arrows ();
+  textat_halign= get_string (TEXTAT_HALIGN);
+  textat_valign= get_string (TEXTAT_VALIGN);
 
   update_src_style ();
   update_src_special ();
@@ -436,21 +525,42 @@ edit_env_rep::update (string s) {
     break;
   case Env_Page:
     break;
+  case Env_Page_Extents:
+    update_page_pars ();
+    break;
   case Env_Preamble:
     preamble= get_bool (PREAMBLE);
     break;
+  case Env_Geometry:
+    update_geometry ();
+    break;
   case Env_Frame:
     update_frame ();
     break;
-  case Env_Clipping:
-    update_clipping ();
-    break;
   case Env_Point_Style:
     point_style= get_string (POINT_STYLE);
     break;
   case Env_Line_Width:
     lw= get_length (LINE_WIDTH);
     break;
+  case Env_Dash_Style:
+    update_dash_style();
+    break;
+  case Env_Dash_Style_Unit:
+    dash_style_unit= get_length (DASH_STYLE_UNIT);
+    break;
+  case Env_Fill_Color:
+    update_color ();
+    break;
+  case Env_Line_Arrows:
+    update_line_arrows();
+    break;
+  case Env_Textat_Halign:
+    textat_halign= get_string (TEXTAT_HALIGN);
+    break;
+  case Env_Textat_Valign:
+    textat_valign= get_string (TEXTAT_VALIGN);
+    break;
   case Env_Src_Style:
     update_src_style ();
     break;
