@@ -20,10 +20,13 @@
 #include "merge_sort.hpp"
 #include "scheme.hpp"
 
+string PS_CLIP_PUSH ("gsave");
+string PS_CLIP_POP ("grestore");
 string PS_CLIP ("cl");
 string PS_LINE ("ln");
 string PS_FILL ("fl");
 string PS_ARC ("ac");
+string PS_FILL_ARC ("fac");
 string PS_POL_START ("sp");
 string PS_POL_NEXT ("np");
 string PS_POL_END ("ep");
@@ -46,7 +49,7 @@ printer_rep::printer_rep (
     defs ("?"), tex_chars ("?"), tex_width ("?"),
     tex_fonts ("?"), tex_font_chars (array<int>(0))    
 {
-  true_type = use_tt_fonts ();
+  type_1    = get_font_type () > 0;
 
   black     = dis->black;
   white     = dis->white;
@@ -67,8 +70,6 @@ printer_rep::printer_rep (
   load_string ("$TEXMACS_PATH/misc/convert/special.pro", special_pro, true);
   load_string ("$TEXMACS_PATH/misc/convert/color.pro", color_pro, true);
   load_string ("$TEXMACS_PATH/misc/convert/texps.pro", texps_pro, true);
-  set_clipping (0, (int) (-(dpi*PIXEL*paper_h)/2.54),
-		(int) ((dpi*PIXEL*paper_w)/2.54), 0);
   
   prologue   << "%!PS-Adobe-2.0\n"
 	     << "%%Creator: TeXmacs-" TEXMACS_VERSION "\n"
@@ -106,6 +107,8 @@ printer_rep::printer_rep (
 	  string ("pt3 pt4 lineto pt1 pt4 lineto pt1 pt2 eofill stroke"));
   define (PS_ARC, string ("/a2 X /a1 X /r2 X /r1 X /pt2 X /pt1 X\n") *
 	  string ("newpath pt1 pt2 r1 r2 a1 a2 ellipse stroke"));
+  define (PS_FILL_ARC, string ("/a2 X /a1 X /r2 X /r1 X /pt2 X /pt1 X\n") *
+	  string ("newpath pt1 pt2 r1 r2 a1 a2 ellipse eofill stroke"));
   define (PS_POL_START, string ("/pt2 X /pt1 X\n") *
 	  string ("newpath pt1 pt2 moveto"));
   define (PS_POL_NEXT, string ("/pt2 X /pt1 X\n") *
@@ -141,15 +144,11 @@ printer_rep::~printer_rep () {
   save_string (ps_file_name, ps_text);
 }
 
-int
-printer_rep::get_type () {
-  return PS_DEVICE_PRINTER;
+bool
+printer_rep::is_printer () {
+  return true;
 }
 
-/******************************************************************************
-* subroutines for printing
-******************************************************************************/
-
 void
 printer_rep::next_page () {
   if (cur_page > 0) print ("eop\n");
@@ -160,6 +159,9 @@ printer_rep::next_page () {
        << as_string (cur_page) << " "
        << as_string (cur_page-1) << " bop\n";
 
+  set_clipping (0, (int) (-(dpi*PIXEL*paper_h)/2.54),
+		(int) ((dpi*PIXEL*paper_w)/2.54), 0);
+
   fg  = -1;
   bg  = -1;
   lw  = -1;
@@ -168,6 +170,10 @@ printer_rep::next_page () {
   ypos= 0;
 }
 
+/******************************************************************************
+* subroutines for printing
+******************************************************************************/
+
 void
 printer_rep::define (string s, string defn) {
   if (defs->contains (s)) return;
@@ -314,6 +320,7 @@ static char* hex_string= "0123456789ABCDEF";
 
 void
 printer_rep::make_tex_char (string name, unsigned char c, glyph gl) {
+  // cout << "Make char " << (int) c << " of " << name << "\n";
   string char_name (name * "-" * as_string ((int) c));
   if (tex_chars->contains (char_name)) return;
   if (!tex_fonts->contains (name)) {
@@ -359,6 +366,21 @@ printer_rep::make_tex_char (string name, unsigned char c, glyph gl) {
   tex_width (char_name)= as_string (d5);
 }
 
+static string
+find_ps_font_name (string name, string s) {
+  int i, n= N(s);
+  for (i=0; i<n; i++) {
+    if (test (s, i, "/FontName /")) {
+      i += 11;
+      int start= i;
+      while (i<n && s[i] != ' ') i++;
+      return s (start, i);
+    }
+    while (i<n && s[i] != '\12' && s[i] != '\15') i++;
+  }
+  return name;
+}
+
 void
 printer_rep::generate_tex_fonts () {
   hashset<string> done;
@@ -380,12 +402,10 @@ printer_rep::generate_tex_fonts () {
       if (suffix (u) == "pfb")
 	ttf= eval_system ("pfbtops", u);
     }
-    else if (true_type && (pos!=-1) && ends (fn_name, "pk"))
-      ttf= pk_to_true_type (root);
 #endif
 
     if (ttf != "") {
-      root= upcase_all (root);
+      string ttf_name= find_ps_font_name (root, ttf);
       if (!done->contains (root)) {
 	prologue << "%%BeginFont: " << root << "\n";
 	prologue << ttf;
@@ -412,7 +432,7 @@ printer_rep::generate_tex_fonts () {
 
       string fdef;
       for (i=N(cum)-1; i>=0; i--) fdef << cum[i];
-      fdef= "/" * name * " " * fdef * " " * mag * " /" * root * " rf";
+      fdef= "/" * name * " " * fdef * " " * mag * " /" * ttf_name * " rf";
       for (i=0, l=0; i<N(fdef); i++, l++)
 	if ((l<70) || (fdef[i]!=' ')) prologue << fdef[i];
 	else { prologue << '\n'; l=-1; }
@@ -451,12 +471,19 @@ printer_rep::generate_tex_fonts () {
 ******************************************************************************/
 
 void
-printer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2) {
+printer_rep::set_clipping (SI x1, SI y1, SI x2, SI y2, bool restore) {
   outer_round (x1, y1, x2, y2);
   ps_device_rep::set_clipping (x1, y1, x2, y2);
-  print (x1, y1);
-  print (x2, y2);
-  print (PS_CLIP);
+  if (restore) {
+    print (PS_CLIP_POP);
+    cfn= "";
+  }
+  else {
+    print (PS_CLIP_PUSH);
+    print (x1, y1);
+    print (x2, y2);
+    print (PS_CLIP);
+  }
 }
   
 /******************************************************************************
@@ -507,7 +534,6 @@ printer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
   if (nil (gl)) return;
   string name= fn->res_name;
   unsigned char c= ch;
-  if (true_type) ec_to_cm (name, c);
   make_tex_char (name, c, gl);
   select_tex_font (name);
   move_to (x, y);
@@ -517,8 +543,9 @@ printer_rep::draw (int ch, font_glyphs fn, SI x, SI y) {
 }
 
 void
-printer_rep::set_line_style (SI w, int type) {
+printer_rep::set_line_style (SI w, int type, bool round) {
   (void) type;
+  (void) round;
   if (lw == w) return;
   lw= w;
   select_line_width (w);
@@ -560,7 +587,17 @@ printer_rep::arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
 }
 
 void
-printer_rep::polygon (array<SI> x, array<SI> y) {
+printer_rep::fill_arc (SI x1, SI y1, SI x2, SI y2, int alpha, int delta) {
+  print ((x1+x2)/2, (y1+y2)/2);
+  print (as_string ((x2-x1)/(2*PIXEL)));
+  print (as_string ((y1-y2)/(2*PIXEL)));
+  print (as_string (((double) alpha)/64));
+  print (as_string (((double) (alpha+delta))/64));
+  print (PS_FILL_ARC);
+}
+
+void
+printer_rep::polygon (array<SI> x, array<SI> y, bool convex) {
   int i, n= N(x);
   if ((N(y) != n) || (n<1)) return;
   print (x[0], y[0]);
@@ -597,10 +634,17 @@ incorporate_postscript (string s) {
 */
 
 void
-printer_rep::postscript (
-  url image, SI w, SI h, SI x, SI y,
-  int x1, int y1, int x2, int y2)
+printer_rep::image (
+  url u, SI w, SI h, SI x, SI y,
+  double cx1, double cy1, double cx2, double cy2)
 {
+  int bx1, by1, bx2, by2;
+  ps_bounding_box (u, bx1, by1, bx2, by2);
+  int x1= bx1 + (int) (cx1 * (bx2 - bx1) + 0.5);
+  int y1= by1 + (int) (cy1 * (by2 - by1) + 0.5);
+  int x2= bx1 + (int) (cx2 * (bx2 - bx1) + 0.5);
+  int y2= by1 + (int) (cy2 * (by2 - by1) + 0.5);
+
   double sc_x= (72.0/dpi) * ((double) (w/PIXEL)) / ((double) (x2-x1));
   double sc_y= (72.0/dpi) * ((double) (h/PIXEL)) / ((double) (y2-y1));
   cr ();
@@ -642,8 +686,8 @@ printer_rep::postscript (
   /* @beginspecial 0 @llx 0 @lly 613.291260 @urx 613.291260 @ury 6110 @rwi
      @clip @setspecial */
   
-  string ps_image= ps_load (image);
-  string imtext= is_ramdisc (image)? "inline image": as_string (image);
+  string ps_image= ps_load (u);
+  string imtext= is_ramdisc (u)? "inline image": as_string (u);
   body << "%%BeginDocument: " << imtext  << "\n";
   body << ps_image; // incorporate_postscript (ps_image);
   body << "%%EndDocument";
@@ -675,10 +719,30 @@ printer_rep::postscript (
   (void) w; (void) h;
 }
 
-bool
-printer_rep::check_event (int type) {
-  (void) type;
-  return false;
+void
+printer_rep::fetch (SI x1, SI y1, SI x2, SI y2, ps_device dev, SI x, SI y) {
+  (void) x1; (void) y1; (void) x2; (void) y2;
+  (void) dev; (void) x; (void) y;
+}
+
+void
+printer_rep::new_shadow (ps_device& dev) {
+  (void) dev;
+}
+
+void
+printer_rep::delete_shadow (ps_device& dev) {
+  (void) dev;
+}
+
+void
+printer_rep::get_shadow (ps_device dev, SI x1, SI y1, SI x2, SI y2) {
+  (void) dev; (void) x1; (void) y1; (void) x2; (void) y2;
+}
+
+void
+printer_rep::put_shadow (ps_device dev, SI x1, SI y1, SI x2, SI y2) {
+  (void) dev; (void) x1; (void) y1; (void) x2; (void) y2;
 }
 
 void
